/***************************************************************************
 * glPresent                                                               *
 * An OpenGL presentation manager                                          *
 * Copyright (C) 2004 Joshua Wise <joshua@joshuawise.com>.                 *
 *                                                                         *
 * This is my code. Don't use it without letting people have at least my   *
 * source. Diffs against my source would be appreciated. Try not to remove *
 * my copyright notice from the startup message, or if you have to, I      *
 * guess move it to the usage message. I won't sue you for copyright       *
 * infringement if you stick to these guidelines. Thanks.                  *
 ***************************************************************************/
#include "bluecubeslide.h"
#include <GL/gl.h>
#include <GL/glut.h>
#include <math.h>
#include <sys/timeb.h>
#include <stdio.h>

BlueCubeSlide::BlueCubeSlide()
 : Slide()
{
}


BlueCubeSlide::~BlueCubeSlide()
{
}

/*!
    \fn BlueCubeSlide::render()
 */
void BlueCubeSlide::render()
{
  struct timeb t;
  static int starttm = 0;
  ftime(&t);
  if (!starttm)
    starttm = t.time;
    
  // background
  glBegin(GL_QUADS);
    glColor3f(0.3f, 0.8f, 0.8f);  glVertex3f(-1.0f, -1.0f, 0.0f);
    glColor3f(0.2f, 0.2f, 0.9f);  glVertex3f(1.0f, -1.0f, 0.0f);
    glColor3f(0.0f, 0.0f, 1.0f);  glVertex3f(1.0f, 1.0f, 0.0f);
    glColor3f(0.2f, 0.2f, 0.9f);  glVertex3f(-1.0f, 1.0f, 0.0f);
  glEnd();
  
  // rotating cube in upper left corner
  float f = t.millitm+(t.time-starttm)*1000;
  f /= 2000.0f;
  glPushMatrix();
    glTranslatef(-0.8f, -0.8f, 0.0f);
    glRotatef(sin(f)*360.0f, 0.0f, 0.0f, 1.0f);
    glRotatef(f*360.0f,      0.0f, 1.0f, 0.0f);
    glRotatef(cos(f)*360.0f, 1.0f, 0.0f, 0.0f);
    glColor4f(0.3f, 0.3f, 1.0f, 1.0f);
    glLineWidth(2.0f);
    glutWireCube(0.2f);
  glPopMatrix();
  
  // Very large heavily alphaed cube
  glPushMatrix();
    glTranslatef(0.1f, 0.2f, 0.0f);
    glRotatef(-f/10*360.0f,  0.0f, 0.0f, 1.0f);
    glRotatef(f/9*360.0f,  0.0f, 1.0f, 0.0f);
    glRotatef(f/8*360.0f, 1.0f, 0.0f, 0.0f);
    glColor4f(0.1f, 0.1f, 0.8f, 0.1f);
    glLineWidth(10.0f);
    glutWireCube(0.95f);
  glPopMatrix();
  
  Slide::render();
}

void BlueCubeSlide::enter(TransitionManager* t)
{
  RenderList* rl = rlist;
  
  while (rl->next)
  {
    rl->object->slideEnter(t);
    rl = rl->next;
  }
}

void BlueCubeSlide::leave(TransitionManager* t)
{
  RenderList* rl = rlist;
  
  while (rl->next)
  {
    rl->object->slideLeave(t);
    rl = rl->next;
  }
}
