/***************************************************************************
 * glPresent                                                               *
 * An OpenGL presentation manager                                          *
 * Copyright (c) 2004 Joshua Wise <joshua@joshuawise.com>. All rights      *
 * reserved.                                                               *
 *                                                                         *
 * This is my code. Don't use it without letting people have at least my   *
 * source. Diffs against my source would be appreciated. Try not to remove *
 * my copyright notice from the startup message, or if you have to, I      *
 * guess move it to the usage message. I won't sue you for copyright       *
 * infringement if you stick to these guidelines. Thanks.                  *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <math.h>
#include "renderable.h"
#include "renderlist.h"
#include "spinningcube.h"
#include "translation.h"
#include "bluecubeslide.h"
#include "bluecubetitle.h"
#include "redcubetitle.h"
#include "slideintransition.h"
#include "slideouttransition.h"
#include "slidepresenter.h"
#include "bluecubelargecontentbox.h"
#include "redcubelargecontentbox.h"
#include "rotateintransition.h"
#include "rotateouttransition.h"

using namespace std;

SlidePresenter* maincontainer;

void idle()
{
  int e;
  
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);
  glEnable(GL_BLEND); 
  glDisable(GL_ALPHA_TEST);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  
  glClearDepth(5.0f);
  glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-1.0, 1.0, 1.0, -1.0, -1.0, 1.0);
  glMatrixMode(GL_MODELVIEW);
  
  maincontainer->render();
  //glFinish();
  glutSwapBuffers();
  
  e = glGetError();
  if (e)
    printf("Error: %08x\n", e);
}

int main(int argc, char *argv[])
{
  int mainwnd;
  Slide* slide;
  Content* content;
    
  printf("glPresent\n");
  printf("Copyright (c) 2004 Joshua Wise <joshua@joshuawise.com>. All rights reserved.\n");
  printf("\n");
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH | GLUT_ALPHA);
  glutInitWindowSize(640,480);
  glutInitWindowPosition(320,240);
  mainwnd = glutCreateWindow("glPresent");
  glutIdleFunc(idle);
  
  maincontainer = new SlidePresenter();
  
  slide = new BlueCubeSlide();
  slide->add(new SlideInTransition(new SlideOutTransition(new BlueCubeTitle("Adam Smith"))));
  content = new BlueCubeLargeContentBox();
  content->addBullet("*WHO IS THIS GUY??");
  content->addBullet("*Scottish economist");
  content->addBullet("*Born 1723");
  content->addBullet("*Died 1790");
  content->addBullet("*So when was that?");
  content->addBullet("*RIGHT IN THE MIDDLE OF");
  content->addBullet(" THE ENLIGHTENMENT!");
  slide->add(new RotateInTransition(new RotateOutTransition(content)));
  maincontainer->add(slide);

  slide = new BlueCubeSlide();
  slide->add(new SlideInTransition(new SlideOutTransition(new BlueCubeTitle("WHO CARES?!"))));
  content = new BlueCubeLargeContentBox();
  content->addBullet("*Scottish economist???");
  content->addBullet("*Who cares?");
  content->addBullet("*Yeah.");
  content->addBullet("*That's what we thought, too.");
  content->addBullet("*But...");
  slide->add(new RotateInTransition(new RotateOutTransition(content)));
  maincontainer->add(slide);
  
  slide = new BlueCubeSlide();
  slide->add(new SlideInTransition(new SlideOutTransition(new BlueCubeTitle("A big deal!!"))));
  content = new BlueCubeLargeContentBox();
  content->addBullet("*He was a MEGA BIG DEAL!");
  content->addBullet("*He brought on the ideas of");
  content->addBullet(" economics as we know them");
  content->addBullet(" today!");
  content->addBullet("*Uhh... What are those?");
  slide->add(new RotateInTransition(new RotateOutTransition(content)));
  maincontainer->add(slide);
  
  slide = new BlueCubeSlide();
  slide->add(new SlideInTransition(new SlideOutTransition(new BlueCubeTitle("Capitalism"))));
  content = new BlueCubeLargeContentBox();
  content->addBullet("*You know, you get money for");
  content->addBullet(" things you do.");
  content->addBullet("*What, you thought it was always");
  content->addBullet(" like that?");
  content->addBullet("*Before our friend Adam, we");
  content->addBullet(" had...");
  slide->add(new RotateInTransition(new RotateOutTransition(content)));
  maincontainer->add(slide);
  
  slide = new BlueCubeSlide();
  slide->add(new SlideInTransition(new SlideOutTransition(new RedCubeTitle("Communism"))));
  content = new RedCubeLargeContentBox();
  content->addBullet("*You've all heard Vadim call");
  content->addBullet(" someone a \"commie\" - but ");
  content->addBullet(" what does it mean??");
  content->addBullet("*The government buys everything");
  content->addBullet(" from you for one price and gives");
  content->addBullet(" it to everyone - regardless of");
  content->addBullet(" how hard they work.");
  content->addBullet("*This pissed people off.");
  slide->add(new RotateInTransition(new RotateOutTransition(content)));
  maincontainer->add(slide);
  
  slide = new BlueCubeSlide();
  slide->add(new SlideInTransition(new SlideOutTransition(new BlueCubeTitle("Capitalism vs. Communism"))));
  content = new BlueCubeLargeContentBox();
  content->addBullet("*Up to now, we've given anybody");
  content->addBullet(" money for anything - regardless");
  content->addBullet(" of whether it was on topic - or");
  content->addBullet(" even correct! THAT'S NO FAIR!");
  content->addBullet("*So now, you get more money");
  content->addBullet(" when you say something ");
  content->addBullet(" insightful. THAT'S MORE FAIR!");
  slide->add(new RotateInTransition(new RotateOutTransition(content)));
  maincontainer->add(slide);
  
  slide = new BlueCubeSlide();
  slide->add(new SlideInTransition(new SlideOutTransition(new BlueCubeTitle("Discussion"))));
  content = new BlueCubeLargeContentBox();
  content->addBullet("*Now that you know the ideas");
  content->addBullet(" of our buddy Mr. Smith, we'll");
  content->addBullet(" stop talking at you.");
  content->addBullet("*What do you think about");
  content->addBullet(" capitalism?");
  content->addBullet("*Is it fair? What about the");
  content->addBullet(" poor people caught in a cycle?");
  slide->add(new RotateInTransition(new RotateOutTransition(content)));
  maincontainer->add(slide);
  
  glutMainLoop();
  return 0;
}
