/***************************************************************************
 * glPresent                                                               *
 * An OpenGL presentation manager                                          *
 * Copyright (C) 2004 Joshua Wise <joshua@joshuawise.com>.                 *
 *                                                                         *
 * This is my code. Don't use it without letting people have at least my   *
 * source. Diffs against my source would be appreciated. Try not to remove *
 * my copyright notice from the startup message, or if you have to, I      *
 * guess move it to the usage message. I won't sue you for copyright       *
 * infringement if you stick to these guidelines. Thanks.                  *
 ***************************************************************************/
#include "rotateintransition.h"
#include <stdio.h>
#include <GL/gl.h>
#include <sys/timeb.h>

RotateInTransition::RotateInTransition(Transition* t): BasicTransition(t)
{
  starttime = 0.0;
}


RotateInTransition::RotateInTransition(Renderable* r): BasicTransition(r)
{
  starttime = 0.0;
}


RotateInTransition::~RotateInTransition()
{
}


void RotateInTransition::slideEnter(TransitionManager* t)
{
  struct timeb tm;
  printf("RotateInTransition::slideEnter(): hit\n");
  ftime(&tm);
  starttime = ((double)tm.time) + ((double)tm.millitm)/1000.0;
  if (t)
    t->start();
  BasicTransition::slideEnter(t);
}

void RotateInTransition::render()
{
  struct timeb tm;
  double thetime;
  if (starttime == 0.0)
  {
    BasicTransition::render();
    return;
  }
  ftime(&tm);
  thetime = ((double)tm.time) + ((double)tm.millitm)/1000.0 - starttime;
  glPushMatrix();
    if (thetime <= 2.0)
      glRotatef(90.0 - thetime*45.0, 1.0f, 0.0f, 0.0f);      
    BasicTransition::render();
  glPopMatrix();
  if (thetime > 2.0)
  {
    printf("RotateInTransition::render(): Done.\n");
    if (manager)
    {
      printf("RotateInTransition::render(): Reporting completion to manager.\n");
      manager->done();
    }
    starttime = 0.0;
  }
}

