/***************************************************************************
 * glPresent                                                               *
 * An OpenGL presentation manager                                          *
 * Copyright (C) 2004 Joshua Wise <joshua@joshuawise.com>.                 *
 *                                                                         *
 * This is my code. Don't use it without letting people have at least my   *
 * source. Diffs against my source would be appreciated. Try not to remove *
 * my copyright notice from the startup message, or if you have to, I      *
 * guess move it to the usage message. I won't sue you for copyright       *
 * infringement if you stick to these guidelines. Thanks.                  *
 ***************************************************************************/
#include "slideintransition.h"
#include <sys/timeb.h>
#include <math.h>
#include <stdio.h>
#include <GL/gl.h>

SlideInTransition::SlideInTransition(Transition* t): BasicTransition(t)
{
  starttime = 0.0;
}


SlideInTransition::SlideInTransition(Renderable* r): BasicTransition(r)
{
  starttime = 0.0;
}


SlideInTransition::~SlideInTransition()
{
}


void SlideInTransition::render()
{
  struct timeb tm;
  double thetime;
  if (starttime == 0.0)
  {
    BasicTransition::render();
    return;
  }
  ftime(&tm);
  thetime = ((double)tm.time) + ((double)tm.millitm)/1000.0 - starttime;
  glPushMatrix();
    if (thetime < 2.0)
      glTranslatef(1.0f + sin((thetime+1.0)/2.0*M_PI), 0.0f, 0.0f);      
    BasicTransition::render();
  glPopMatrix();
  if (thetime > 2.0)
  {
    printf("SlideInTransition::render(): Done.\n");
    if (manager)
    {
      printf("SlideInTransition::render(): Reporting completion to manager.\n");
      manager->done();
    }
    starttime = 0.0;
  }
}

void SlideInTransition::slideEnter(TransitionManager* t)
{
  struct timeb tm;
  printf("SlideInTransition::slideEnter(): hit\n");
  ftime(&tm);
  starttime = ((double)tm.time) + ((double)tm.millitm)/1000.0;
  if (t)
    t->start();
  BasicTransition::slideEnter(t);
}

