/***************************************************************************
 * glPresent                                                               *
 * An OpenGL presentation manager                                          *
 * Copyright (C) 2004 Joshua Wise <joshua@joshuawise.com>.                 *
 *                                                                         *
 * This is my code. Don't use it without letting people have at least my   *
 * source. Diffs against my source would be appreciated. Try not to remove *
 * my copyright notice from the startup message, or if you have to, I      *
 * guess move it to the usage message. I won't sue you for copyright       *
 * infringement if you stick to these guidelines. Thanks.                  *
 ***************************************************************************/
#include "slidepresenter.h"
#include <stdlib.h>
#include <GL/glut.h>
#include <stdio.h>

static SlidePresenter* mainpresenter;
static void gmousefunc(int a, int b, int c, int d);

SlidePresenter::SlidePresenter()
{
  mainpresenter = this;
  cur = this;
  glutMouseFunc(gmousefunc);
  slide = NULL;
  prev = NULL;
  next = NULL;
  manager = new TransitionManager();
  isTransitioning = 0;
}

SlidePresenter::~SlidePresenter()
{
  if (prev)
    prev->next = next;
  if (next)
    next->prev = prev;
  delete manager;
}

static void gmousefunc(int button, int state, int x, int y)
{
  mainpresenter->mousefunc(button, state, x, y);
}

void SlidePresenter::render()
{
  cur->slide->render();
  if (isTransitioning && manager->isIdle())
  {
    printf("SlidePresenter::render(): Transition complete!\n");
    isTransitioning--;
    if (cur->next->slide)
      cur = cur->next;
    else
      exit(0);
    printf("SlidePresenter::render(): Entering next slide\n");
    cur->slide->enter(NULL);
  }
}

void SlidePresenter::add(Slide* s)
{
  SlidePresenter* p = this;
  while (p->next) p = p->next;
  p->slide = s;
  p->next = new SlidePresenter();
  mainpresenter = this; // asserting my authoritay: I am the main slidepresenter instance.
  if (p == this) // special case -- this is the first SlidePresenter
    slide->enter(manager);
}

void SlidePresenter::mousefunc(int button, int bstate, int x, int y)
{
  // react on button release
  if (bstate != GLUT_UP)
    return; // who cares?
  if (manager->isIdle() && !isTransitioning && cur->slide->event())
  {
    isTransitioning++;
    printf("SlidePresenter::mousefunc(int, int, int, int): Leaving slide with manager %08x (%sidle)\n", manager, manager->isIdle() ? "" : "not ");
    cur->slide->leave(manager);
  }
}
